@@ -7,6 +7,11 @@ our (@EXPORT,%EXPORT_TAGS,@EXPORT_OK);
 
 my @logfuncs;
 
+sub log_dummy {
+    @_ = sprintf("".shift @_, @_);
+    goto &carp;
+}
+
 $SIG{__DIE__} = \&confess;
 
 BEGIN {
@@ -16,7 +21,7 @@ BEGIN {
 
 use Module::Stubber
     'Log::Fu' => [ { level => "debug" } ],
-    will_use => { map { $_ => \&carp } @logfuncs };
+    will_use => { map { $_ => \&log_dummy } @logfuncs };
 
 #Keep these in sync with hrdefs.h HR_HKEY_LOOKUP_
 use Constant::Generate [qw(
@@ -30,8 +35,23 @@ use Constant::Generate [qw(
     HR_TIDX_UNKEYFUNC
     HR_TIDX_FLAGS
     HR_TIDX_PRIVDATA
+    HR_TIDX_ITER
 )], export => 1;
 
+#These map method names to lookup indicex
+our %LookupNames = (
+    reverse         => HR_TIDX_RLOOKUP,
+    scalar_lookup   => HR_TIDX_SLOOKUP,
+    forward         => HR_TIDX_FLOOKUP,
+    attr_lookup     => HR_TIDX_ALOOKUP,
+    keyfunc         => HR_TIDX_KEYFUNC,
+    unkeyfunc       => HR_TIDX_UNKEYFUNC,
+    keytypes        => HR_TIDX_KEYTYPES,
+    priv            => HR_TIDX_PRIVDATA,
+    flags           => HR_TIDX_FLAGS,
+    _iter           => HR_TIDX_ITER
+);
+
 use Constant::Generate [qw(
     HR_KFLD_STRSCALAR
     HR_KFLD_REFSCALAR
@@ -52,10 +72,15 @@ use Constant::Generate [qw(
     DUPIDX_SLOOKUP
 )], -tag => 'pp_constants' => -export_ok => 1;
 
+#Keep this in sync with C code
+use Constant::Generate {
+    HR_PREFIX_DELIM => '#'
+}, export => 1;
+
 BEGIN {
     if(!$Module::Stubber::Status{'Log::Fu'}) {
         push @EXPORT, @logfuncs;
     }
 }
 
-1;
\ No newline at end of file
+1;
